package items;

import pokemon.Pokemon;

/**
 * An abstract class representing a potion
 */
public abstract class AbstractPotion extends Item {
    final boolean alive;

    /**
     * Instantiates a new potion
     * @param name <p>The name of the potion</p>
     * @param description <p>The description of the potion</p>
     */
    public AbstractPotion(String name, String description) {
        super(name, description);
        this.alive = true;
    }

    /**
     * Instantiates a new potion
     * @param name <p>The name of the potion</p>
     * @param description <p>The description of the potion</p>
     * @param alive <p>Whether the pokemon needs to be alive to use the potion</p>
     */
    public AbstractPotion(String name, String description, boolean alive) {
        super(name, description);
        this.alive = alive;
    }

    /** Checks if an item should be used on alive or fainted pokemon. */
    public boolean needsAlive() {
        return alive;
    }

    /**
     * Uses an item and does something based on the type of item
     * @param target <p>Which pokemon should the item be used on</p>
     * @return <p>True if nothing went wrong. False otherwise</p>
     */
    public boolean use(Pokemon target) {
        return false;
    }

    /**
     * Checks if a pokemon is able to, and in need of a potion. If it is, heal it
     * @param target <p>The pokemon to heal</p>
     * @param amount <p>The amount to heal the pokemon</p>
     * @return <p>True if nothing went wrong. False otherwise</p>
     */
    boolean potion(Pokemon target, int amount) {
        if (target.isConscious()) {
            if (target.isDamaged()) {
                int healed = target.heal(amount);
                System.out.printf("%s was healed for %d HP and now has %d HP.%n", target.getName(), healed, target.getHP());
                return true;
            } else {
                System.out.printf("%s has not taken damage, and does not require a potion.%n", target.getName());
                return false;
            }
        } else {
            System.out.println("You can't heal a fainted pokemon.");
            return false;
        }
    }
}
