package storage;

import pokemon.Pokemon;

import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

/**
 * This class represents a pokemon trainer
 */
public class Trainer {
    public static final Scanner in = new Scanner(System.in);
    final String name;
    private List<Pokemon> pokemon;
    private Inventory inventory;

    /**
     * Instantiates a new trainer
     * @param name <p>The name of the trainer</p>
     * @param pokemon <p>The pokemon belonging to the trainer</p>
     * @param inventory <p>The inventory belonging to the trainer</p>
     */
    public Trainer(String name, ArrayList<Pokemon> pokemon, Inventory inventory) {
        this.name = name;
        this.pokemon = pokemon;
        this.inventory = inventory;
    }

    /**
     * Sets the list of pokemon belonging to the trainer
     * @param pokemon <p>A list of pokemon</p>
     */
    public void setPokemon(List<Pokemon> pokemon) {
        this.pokemon = pokemon;
    }

    /**
     * Sets the inventory belonging to the trainer
     * @param inventory <p>An inventory</p>
     */
    public void setInventory(Inventory inventory) {
        this.inventory = inventory;
    }

    /**
     * Gets the list of pokemon belonging to the trainer
     * @return <p>A list of pokemon</p>
     */
    public List<Pokemon> getPokemon() {
        return this.pokemon;
    }

    /**
     * Adds a pokemon to the trainer's list of pokemon
     * @param pokemon <p>A pokemon</p>
     */
    public void addPokemon(Pokemon pokemon) {
        this.pokemon.add(pokemon);
    }

    /**
     * Gets all conscious pokemon belonging to the trainer
     * @return <p>A list of pokemon</p>
     */
    public ArrayList<Pokemon> getConsciousPokemon() {
        ArrayList<Pokemon> pokemon = new ArrayList<>();
        for (Pokemon singlePokemon : this.pokemon) {
            if (singlePokemon.isConscious()) {
                pokemon.add(singlePokemon);
            }
        }
        return pokemon;
    }

    /**
     * Gets all fainted pokemon belonging to the trainer
     * @return <p>A list of pokemon</p>
     */
    public ArrayList<Pokemon> getFaintedPokemon() {
        ArrayList<Pokemon> pokemon = new ArrayList<>();
        for (Pokemon singlePokemon : this.pokemon) {
            if (!singlePokemon.isConscious()) {
                pokemon.add(singlePokemon);
            }
        }
        return pokemon;
    }

    /**
     * Gets the inventory of the user
     * @return <p>An inventory</p>
     */
    public Inventory getInventory() {
        return this.inventory;
    }

    /**
     * Lists all currently available pokemon for the trainer
     * @param alive <p>Whether to list alive pokemon or dead</p>
     */
    public void listAvailablePokemon(boolean alive) {
        ArrayList<Pokemon> pokemonList;
        if (alive) {
            pokemonList = this.getConsciousPokemon();
        } else {
            pokemonList = this.getFaintedPokemon();
        }
        System.out.println("You may choose from these pokemon:");
        for (int i = 0; i < pokemonList.size(); i++) {
            System.out.printf("%d: %s%n", i + 1, pokemonList.get(i));
        }
        System.out.print(">");
    }

    /**
     * Prints all pokemon belonging to the trainer
     */
    public void printPokemon() {
        for (Pokemon pokemon : this.pokemon) {
            System.out.println(pokemon);
        }
    }

    /**
     * Asks the user for the name of a pokemon and returns it
     * @param alive <p>Whether the chosen pokemon has to be alive</p>
     * @return <p>A pokemon object or null</p>
     */
    public Pokemon choosePokemon(boolean alive) {
        ArrayList<Pokemon> pokemonList;
        if (alive) {
            pokemonList = this.getConsciousPokemon();
        } else {
            pokemonList = this.getFaintedPokemon();
        }
        if (in.hasNextInt()) {
            int choice = in.nextInt() - 1;
            if (choice >= 0 && choice < pokemonList.size()) {
                in.nextLine();
                return pokemonList.get(choice);
            }
        }
        in.nextLine();
        return null;
    }
}
